var parseOperator = function(op) {
	switch(op) {
		case 'GT':
			return 'GREATER_THAN';
		case 'LT':
			return 'LESS_THAN';
		case 'EQ':
			return 'EQUAL_TO';
		case 'GE':
            return 'GREATER_THAN_EQ';
        case 'LE':
            return 'LESS_THAN_EQ';
        case 'NE':
        	return 'NOT_EQUAL_TO';
        case 'NOT':
        	return 'NOT_OP';
        default:
        	return op;
	}
};

var parseAttribute = function(attr) {
	switch(attr){
		case 'jobTitle':
			return 'JOB_TITLE';
		case 'fullName':
			return 'FULL_NAME';
		case 'displayName':
			return 'DISPLAY_NAME';
		case 'location':
			return 'LOCATION';
		case 'risk':
			return 'RISK';
		case 'jobCode':
			return 'JOB_CODE';
		case 'firstName':
			return 'FIRST_NAME';
		case 'lastName':
			return 'LAST_NAME';
		case 'middleName':
			return 'MIDDLE_NAME';
        case 'dn':
        	return 'LDAP_DISTINGUISHED_NAME';
		case 'supervisorUniqueId':
			return 'MANAGED_BY';
		case 'department':
			return 'DEPARTMENT';
		case 'displayDescription':
			return 'DISPLAY_DESCRIPTION';
		case 'employeeType':
			return 'EMPLOYEE_TYPE';
		case 'employeeStatus':
			return 'EMPLOYEE_STATUS';
		case 'homePhone':
			return 'HOME_PHONE';
		case 'officePhone':
			return 'OFFICE_PHONE';
		case 'userId':
			return 'USER_ID';
		case 'workforceId':
			return 'WORKFORCE_ID';
	}
};

var parseEntities = function(entities) {
	var resultList = [];
	if (entities !== null) {
		entities.forEach(
			function(entity) {
				resultList.push("'" + entity.displayName + "'");
			});
	}
	return resultList.join(', ');
};

var parseValues = function(values) {
	var resultList = [];
	if (values !== null) {
		values.forEach(
			function(value) {
				resultList.push("'" + value + "'");
			});
	}
	return resultList.join(', ');
};

var parseValue = function(value, attribute) {
	// If attribute is not risk
	var resultList = [];
	if (value !== null && attribute !== null) {
		if(attribute === 'risk') {
			resultList.push(value);
		} else {
			resultList.push("'" + value + "'");
		}
	}
	return resultList.join(' ');
};

var parseMemberExpression = function(jsonObj, depth) {
	var resultList = [];
	var startOf = 0;	// Do not add relational ops besides "NOT" to the start of a statement
	var currentDepth = depth + 1;	// If not the base level of logic and has muliple criteria, add parentheses
	var numCriteria = 0;
	if (jsonObj.childCriteria !== undefined && depth !== null && jsonObj.operator !== undefined) {
		var criteriaList = jsonObj.childCriteria;
		criteriaList.forEach(
			function(criteria) {
				if(startOf === 1 && jsonObj.operator !== 'NOT') {
					resultList.push(jsonObj.operator);
				} else if (jsonObj.operator === 'NOT') {
					resultList = resultList.concat(parseOperator(jsonObj.operator));
					resultList = resultList.concat(" (");
				}
				if(criteria.childCriteria !== undefined) {
					resultList = resultList.concat(parseMemberExpression(criteria, currentDepth));
				} else if (criteria.operator !== undefined && criteria.operator !== undefined) {
					resultList.push(parseAttribute(criteria.attributeKey));
					resultList.push(parseOperator(criteria.operator));

					if (criteria.value !== undefined) {
						resultList.push(parseValue(criteria.value, criteria.attributeKey));
					} else if (criteria.values !== undefined) {
						resultList.push(parseValues(criteria.values));
					} else if (criteria.entities !== undefined) {
						resultList.push(parseEntities(criteria.entities));
					}
				}

				if (jsonObj.operator === 'NOT') {
					resultList = resultList.concat(")");
				}
				startOf = 1;
				numCriteria = numCriteria + 1;
			});
	}
	if (numCriteria > 1 && currentDepth > 1) {
		var returnList = [];
		returnList.push('(');
		resultList.push(')');
		returnList = returnList.concat(resultList);
		return returnList;
	} else {
		return resultList;
	}
};

var localizeMemberExpression = function(jsonStr, localizer) {
	var resultList = [];
	var jsonObj = JSON.parse(jsonStr);
	var depth = 0;
	if (jsonObj !== null && localizer !== null) {
		resultList = parseMemberExpression(jsonObj, depth);
		return resultList.map(function(str) {
					return localizer.getString(str);
			}).join(' ');
	}
	return resultList.join(' ');
};